"""This is a dummy file, so that this  directory can be used as a module. The
actual magic should *always* happen in tokenizer.py. Tests.py is a stand-alone
test suite and does, really, tests."""

from . import output, tokenizer
from . import output, output
