typeset -ra sedfiles_ordered=(
	typos.sed
	grammar.sed
	inflected_forms.sed
	usage.sed
	usage_debatable.sed
	slashes.sed
	abbreviations.sed
	misc.sed
	quotes.sed
	additions.sed
	alter.sed
	drop.sed
)
